;
; WebSkel1.asm by Jeremy Gordon May 2007
; This file creates a basic window to hold a WebBrowser control.
; The file WebSkel2.asm holds what appears to be the minimum code required to create
; a WebBrowser control.
;
; Assemble using GoAsm WebSkel1
; This creates WebSkel1.obj
; Then link using .....
; Golink WebSkel1.obj WebSkel2.obj /fo WebSkel.exe /debug coff kernel32.dll user32.dll ole32.dll
;                     (or use a command file eg. GoLink @respons.fil)
;
DATA
;
hInst DD 0            ;to keep the handle to the process itself
MSG DD 7 DUP 0        ;structure to hold messages from Windows as follows:-
;hwnd, +4=message, +8=wParam, +C=lParam, +10h=time, +14h/18=pt
WNDCLASS DD 10D DUP 0 ;structure to send to RegisterClass holding data:-
                      ;+0 window class style (CS_)
                      ;+4 pointer to Window Procedure
                      ;+8 no. of extra bytes to allocate after structure
                      ;+C no. of extra bytes to allocate after window instance
                      ;+10 handle to instance of this window class
                      ;+14 handle to the class icon
                      ;+18 handle to the class cursor
                      ;+1C identifies the class background brush
                      ;+20 pointer to resource name for class menu
                      ;+24 pointer to string for window class name
;******************** Window message table 
;          (in a real program this would deal with many more messages)
MESSAGES DD (ENDOF_MESSAGES-$-4)/8      ;=number to be done
         DD  1h,CREATE,2h,DESTROY
ENDOF_MESSAGES:           ;label used to work out how many messages
;******************************************
;
WINDOW_CLASSNAME DUS 'WB_HOLDER',0  ;unicode string to hold name of window class
FILE_BUFFER DB 600 DUP ?            ;holds MAX_PATH chars (520 bytes) plus 80 bytes for html filename
;
;------------------------------------------------------------------
;
CODE
;
INITIALISE_WNDCLASS:    ;get ready to register the window class
MOV EBX,ADDR WNDCLASS
MOV EAX,9
L0:
MOV D[EBX+EAX*4],0      ;fill it with zeroes
DEC EAX
JNS L0
;***** add things to window class for all windows in the program ..
MOV EAX,[hInst]         ;get handle to the process
MOV [EBX+10h],EAX       ;make it the window class
RET
;
;*******************
CREATE:                 ;one of the few messages dealt with by this prog
MOV EBX,[EBP+8h]        ;get window handle in ebx
CALL EmbedBrowserObject ;create WebBrowser control
JC >L1                  ;failure
XOR EAX,EAX             ;return zero to make window
RET
L1:
XOR EAX,EAX
DEC EAX                 ;return -1 and nc so that CreateWindowEx returns NULL handle
RET
;
DESTROY:                ;one of the few messages dealt with by this prog
MOV EAX,[EBP+8h]        ;get window handle in eax
CALL UnEmbedBrowserObject   ;Destroy WebBrowser control
INVOKE PostQuitMessage,0    ;exit via the message loop
STC                     ;go to DefWindowProc too
RET
;
;********** this is a general window procedure which in an ordinary
;********** program deals with all messages sent to the window
GENERAL_WNDPROC:        ;eax can be used to convey information to the call
PUSH EBP                ;use ebp to avoid using eax which may hold information
MOV EBP,[ESP+10h]       ;uMsg
MOV ECX,[EDX]           ;get number of messages to do
ADD EDX,4               ;jump over size dword
L2:
DEC ECX
JS >L3
CMP [EDX+ECX*8],EBP     ;see if its the correct message
JNZ L2                  ;no
MOV EBP,ESP
PUSH ESP,EBX,EDI,ESI    ;save registers as required by Windows
ADD EBP,4               ;allow for the extra call to here
;now [EBP+8]=hwnd, [EBP+0Ch]=uMsg, [EBP+10h]=wParam, [EBP+14h]=lParam,
CALL [EDX+ECX*8+4]      ;call the correct procedure for the message
POP ESI,EDI,EBX,ESP
JNC >L4                 ;nc=return value in eax - don't call DefWindowProc
L3:
PUSH [ESP+18h],[ESP+18h],[ESP+18h],[ESP+18h]     ;allowing for change of ESP
CALL DefWindowProcW
L4:
POP EBP
RET
;
;******************* This is the actual window procedure
WndProcTable:
MOV EDX,ADDR MESSAGES   ;give edx the list of messages to deal with
CALL GENERAL_WNDPROC    ;call the generic message handler
RET 10h                 ;restore the stack as required by caller
;
;*******************************************************************
;
ERROR_BOX:
PUSH 10h                ;error sign + ok button only
PUSH 'WebSkel failed'
PUSH ESI,0
CALL MessageBoxA        ;wait till ok pressed
RET
;
;*******************************************************************
START:
INVOKE GetModuleHandleW,0   ;get handle to the process
MOV [hInst],EAX         ;record it in data label hInst
INVOKE OleInitialize,0
TEST EAX,EAX
JZ >L10                 ;ok
MOV ESI,ADDR 'Failed to initialise OLE'
CALL ERROR_BOX
JMP >>.exit
L10:
CALL INITIALISE_WNDCLASS    ;get ready to register window class (return in ebx)
;********** now add things specific to the window to be made
MOV D[EBX+4],ADDR WndProcTable          ;window procedure
MOV D[EBX+24h],ADDR WINDOW_CLASSNAME    ;window class name
PUSH EBX                ;address of structure with window class data
CALL RegisterClassW     ;register the window class
PUSH 0,[hInst],0,0      ;owner=desktop
PUSH 560D               ;height
PUSH 840D               ;width
PUSH 50D,50D            ;position y then x
PUSH 90000000h +0C00000h+80000h +20000h     +10000h      ;window style
;(POPUP+VISIBLE)+CAPTION+SYSMENU+MINIMIZEBOX+MAXIMIZEBOX
PUSH L'WebBrowser control'     ;window title
PUSH ADDR WINDOW_CLASSNAME     ;window class name
PUSH 0
CALL CreateWindowExW    ;make window, returning handle in EAX
OR EAX,EAX              ;see if window was made
JZ >L102                ;no, clear up and exit
MOV EBX,EAX             ;keep window handle in ebx
;************************ see now CREATE which makes WebBrowser control
;************************ get full path of WebSkel.html
MOV ESI,ADDR FILE_BUFFER
MOV EDI,ESI
INVOKE GetCurrentDirectoryW,300,ESI    ;size of buffer is in characters
SHL EAX,1               ;each char is two bytes
ADD EDI,EAX             ;get to end of path
MOV ESI,ADDR L"\WebSkel.html"
INVOKE lstrlenW,ESI     ;get length in chars of unicode filename
SHL EAX,1               ;each char is two bytes
MOV ECX,EAX
JECXZ >L98
REP MOVSB               ;add filename to end of current directory path
L98:
;************************ put something in the WebBrowser control
MOV ESI,ADDR FILE_BUFFER
MOV EAX,EBX             ;restore window handle to eax
CALL DisplayHTMLPage    ;call with eax=hWnd, esi=url
;************************ now enter the main message loop
L100:
PUSH 0,0,0
PUSH ADDR MSG
CALL GetMessageW        ;wait for message from Windows
OR EAX,EAX              ;see if it is WM_QUIT
JZ >L102                ;yes
PUSH ADDR MSG
CALL TranslateMessage   ;no so convert message to character if necessary
PUSH ADDR MSG
CALL DispatchMessageW   ;and send the message to the window procedure
JMP L100                ;after message dealt with, loop back for next one
L102:
CALL OleUninitialize
PUSH [hInst],ADDR WINDOW_CLASSNAME
CALL UnregisterClassW   ;ensure class is removed
.exit:
PUSH [MSG+8h]           ;exit code (send contents of wParam)
CALL ExitProcess        ;return to Windows in the manner it prefers
